from django.db import migrations, models

try:
    from citext import CIEmailField
except ImportError:
    CIEmailField = models.EmailField


class Migration(migrations.Migration):
    dependencies = [
        ("mailauth_user", "0004_auto_20200812_0722"),
    ]

    operations = [
        # add new permissions
        migrations.AlterModelOptions(
            name="emailuser",
            options={
                "permissions": [("anonymize", "Can anonymize user")],
                "verbose_name": "user",
                "verbose_name_plural": "users",
            },
        ),
        # email is now nullable
        migrations.AlterField(
            model_name="emailuser",
            name="email",
            field=CIEmailField(
                blank=True,
                db_index=True,
                max_length=254,
                null=True,
                unique=True,
                verbose_name="email address",
            ),
        ),
    ]



from django.dispatch import Signal

anonymize = Signal()
"""
Signal that is emitted when a user and all their data should be anonymized.

The signal is emitted before the private date is delete on the instance,
thus the receiver can still access the data. The receiver should usually
not alter the instance, but only later related data. We recommend overriding
the anonymize method to modify the instance.

Usage::

    from django.dispatch import receiver
    from mailauth.contrib.user.models import EmailUser
    from mailauth.contrib.user.signals import anonymize


    @receiver(anonymize, sender=EmailUser)
    def anonymize_user(sender, instance, **kwargs):
        # Do something with related user data
        instance.related_model.delete()

"""



from django.contrib import admin
from django.contrib.auth.models import Group, Permission
from django.utils.translation import (
    gettext_lazy as _,
    ngettext,
)

from . import models


class AnonymizableAdminMixin:
    """
    Mixin for admin classes that provides a `anonymize` action.

    This mixin calls the `anonymize` method of all user model instances.
    """

    actions = ["anonymize"]

    @admin.action(
        permissions=["anonymize"],
        description=_("Anonymize selected %(verbose_name_plural)s"),
    )
    def anonymize(self, request, queryset):
        count = queryset.count()
        for user in queryset.iterator():
            user.anonymize()

        self.message_user(
            request,
            ngettext(
                "%(count)s %(obj_name)s has successfully been anonymized.",
                "%(count)s %(obj_name)s have successfully been anonymized.",
                count,
            )
            % {
                "count": count,
                "obj_name": (
                    self.model._meta.verbose_name_plural
                    if count > 1
                    else self.model._meta.verbose_name
                ),
            },
            fail_silently=True,
        )

    def has_anonymize_permission(self, request, obj=None):
        return request.user.has_perm(f"{self.opts.app_label}.anonymize", obj=obj)


@admin.register(models.EmailUser)
class EmailUserAdmin(AnonymizableAdminMixin, admin.ModelAdmin):
    list_display = ("email", "first_name", "last_name", "is_staff")
    list_filter = ("is_staff", "is_superuser", "is_active", "groups")
    search_fields = ("first_name", "last_name", "email")
    ordering = ("email",)
    filter_horizontal = (
        "groups",
        "user_permissions",
    )

    fieldsets = (
        (None, {"fields": (("email", "is_active"), ("first_name", "last_name"))}),
        (
            Group._meta.verbose_name_plural,
            {
                "fields": ("groups",),
            },
        ),
        (
            Permission._meta.verbose_name_plural,
            {
                "classes": ("collapse",),
                "fields": (("is_staff", "is_superuser"), "user_permissions"),
            },
        ),
    )
